----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:26:59 09/26/2009 
-- Design Name: 
-- Module Name:    intctrl - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity intctrl is
    Port ( CLK : in  STD_LOGIC;
           Reset : in  STD_LOGIC;
           INT : in  STD_LOGIC_VECTOR (3 downto 0);
           INTA : in  STD_LOGIC;
           D : inout  STD_LOGIC_VECTOR (7 downto 0);
           INTR : out  STD_LOGIC);
end intctrl;

architecture Behavioral of intctrl is
	signal lint : STD_LOGIC_VECTOR(INT'range) := (others => '0');
	signal irst : STD_LOGIC_VECTOR(INT'range);
	signal ackid : STD_LOGIC_VECTOR(2 downto 0);
		
	function OR_VECTOR(x : STD_LOGIC_VECTOR)
	return STD_LOGIC is
		variable t : STD_LOGIC;
	begin
		t := '0';
		for i in x'range loop
			t := t or x(i);
		end loop;
		return t;
	end function;
begin

	INTR <= OR_VECTOR(lint);
	D <=	"00101" & ackid when INTA = '1' else	-- ext ints @ 0x28+
			(others => 'Z');

	-- svaki flag zasebno se postavi na falling_edge ulaza, a poništi pri reset-u ili procesiranju
	intbit : for i in INT'range generate
		process (Reset, irst(i), INT(i))
		begin
			if (Reset = '1' or irst(i) = '1') then
				lint(i) <= '0';
			elsif (falling_edge(INT(i))) then
				lint(i) <= '1';
			end if;
		end process;
	end generate intbit;
	
	-- Zapamti koji interrupt treba procesirati dok ga CPU čita
	-- Napomena: Bitno kako interrupt koji nastane za vrijeme čitanja ne bi poremetio situaciju
	process (CLK, INTA)
		variable intid : STD_LOGIC_VECTOR(ackid'range);
		variable llint : STD_LOGIC_VECTOR(lint'range);
	begin
		-- osiguranje da je llint definiran prije rising_edge(INTA):
		if (falling_edge(CLK)) then
			if (INTA = '0') then
				llint := lint;
			end if;
		end if;
		
		-- priority enkodiranje interrupta;
		-- ako niti jedan interrupt nije postavljen, a došlo je do INTA, bit će INTMAX+1
		irst <= (others => '0');
		intid := (others => '0');
		for i in llint'low to llint'high loop
			if (llint(i) = '1') then
				irst(i) <= INTA;
				exit;
			end if;
			intid := intid + 1;
		end loop;
		ackid <= intid;
	end process;

end Behavioral;

