----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:27:50 05/21/2009 
-- Design Name: 
-- Module Name:    selector - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity selector is
	Port (CPUCLK	: in  STD_LOGIC;
			Reset		: in  STD_LOGIC;
			A23		: in  STD_LOGIC;
			A			: in  STD_LOGIC_VECTOR(3 downto 1);
			ADS		: in  STD_LOGIC;
			MIO		: in  STD_LOGIC;
			DC			: in  STD_LOGIC;
			READY		: in  STD_LOGIC;
			BLEn		: in  STD_LOGIC;
			BHEn		: in  STD_LOGIC;
			RDn		: in  STD_LOGIC;
			WRn		: out STD_LOGIC;
			ROME		: out STD_LOGIC;
			RAMLEn	: out STD_LOGIC;
			RAMHEn	: out STD_LOGIC;
			INTA		: out STD_LOGIC;
			REGE		: out STD_LOGIC_VECTOR(4 downto 0));
end selector;

architecture Behavioral of selector is
	type stanje_type is (CIdle, CT1b, CT2a, CT2b);
	signal stanje : stanje_type := CIdle;
	
	signal active : STD_LOGIC;
	signal memrq, ramrq, iorq, ramreg : STD_LOGIC;
	signal intreg : STD_LOGIC_VECTOR(5 downto 0);
	
	signal ramdis : STD_LOGIC := '1';
	
begin

	cycledetector : process (CPUCLK, ADS, READY, Reset, stanje)
	begin
		if (Reset = '1') then
			stanje <= CIdle;
		elsif (rising_edge(CPUCLK)) then
			case stanje is
				when CT1b =>
					stanje <= CT2a;
				when CT2a =>
					stanje <= CT2b;
				when CT2b =>
					if (READY = '1') then
						stanje <= CIdle;
					else
						stanje <= CT2a;
					end if;
				when others =>
					if (ADS = '0') then
						stanje <= CT1b;
					else
						stanje <= CIdle;
					end if;
			end case;
		end if;
		
		if (stanje /= CIdle) then
			active <= '1';
		else
			active <= '0';
		end if;
	end process;
	
	ramdisreg : process (Reset, ramreg)
	begin
		if (Reset = '1') then
			ramdis <= '1';
		elsif (falling_edge(ramreg)) then
			ramdis <= '0';
		end if;
	end process;

	WRn <= not RDn;

	memrq <= MIO and active;
	ROME <= (A23 or ramdis) and memrq;
	ramrq <= (not A23 and not ramdis) and memrq;
	RAMLEn <= not (ramrq and not BLEn);
	RAMHEn <= not (ramrq and not BHEn);

	iorq <= (not MIO) and DC and active;
	INTA <= (not MIO) and (not DC) and (not A(2)) and active;	-- WR nije bitan, A2=0

	with A & iorq select
		intreg <=	"000001" when "0001",
						"000010" when "0011",
						"000100" when "0101",
						"001000" when "0111",
						"010000" when "1001",
						"100000" when "1011",
						"000000" when others;
	REGE <= intreg(5 downto 1);
	ramreg <= intreg(0);

end Behavioral;

