----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:05:22 09/16/2009 
-- Design Name: 
-- Module Name:    spi - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity spi is
    Port ( CLK : in  STD_LOGIC;
           RST : in  STD_LOGIC;
           D : inout  STD_LOGIC_VECTOR (7 downto 0);
           DREG : in  STD_LOGIC;
           SREG : in  STD_LOGIC;
           WR : in  STD_LOGIC;
           SCK : out  STD_LOGIC;
           MISO : in  STD_LOGIC;
           MOSI : out  STD_LOGIC);
end spi;

architecture Behavioral of spi is

	signal spickcont : STD_LOGIC;
	signal spick : STD_LOGIC := '1';
	signal start : STD_LOGIC;
	signal busy : STD_LOGIC := '0';
	
	signal reg : STD_LOGIC_VECTOR(7 downto 0);
	signal bitcount : STD_LOGIC_VECTOR(3 downto 0);
	
	signal fastclk : STD_LOGIC := '1';

begin
	start <= WR and DREG;

	-- djeljenje CLK
	-- Napomena: SCK je još jednom /2
	process (CLK, busy, start, fastclk)
		variable diver : STD_LOGIC_VECTOR(1 downto 0);
	begin
		if (busy = '0' or start = '1') then
			diver := (others => '1');	-- tako da je sljedeći falling
		elsif (rising_edge(CLK)) then
			diver := diver + 1;
		end if;

		if (fastclk = '1') then
			spickcont <= diver(0);	-- /2
		else
			spickcont <= diver(diver'high);	-- /4
		end if;
	end process;

	-- reset, shift & write
	process (RST, start, D, busy, spickcont)
	begin
		if (RST = '1') then
			busy <= '0';
			spick <= '1';
			MOSI <= 'Z';
		elsif (start = '1') then
			busy <= '1';
			reg <= D;
			MOSI <= D(7);
			bitcount <= "0000";
			spick <= '0';
		elsif (rising_edge(spickcont)) then	-- spickcont čeka da nestane start
			if (busy = '1') then
				spick <= not bitcount(0);
				
				if (spick = '1') then	-- promjena MOSI na padajući brid spick
					MOSI <= reg(7);
				else							-- čitanje MISO na rastući
					reg <= reg(6 downto 0) & MISO;
				end if;
				
				if (bitcount = "1111") then
					busy <= '0';
					spick <= '1';	-- override u zadnjem ciklusu
				end if;

				bitcount <= bitcount + 1;
			end if;
		end if;
	end process;
	
	-- write config
	process (RST, SREG)
	begin
		if (RST = '1') then
			fastclk <= '1';
		elsif (falling_edge(SREG)) then
			if (WR = '1') then
				fastclk <= D(1);
			end if;
		end if;
	end process;

	-- izlazni signali moraju biti Z dok je RST
	--MOSI	<= reg(7)	when RST = '0' else 'Z';
	SCK	<= spick		when RST = '0' else 'Z';

	-- čitanje registra je uvijek moguće
	D <=	("010000" & fastclk & busy)	when SREG = '1' and WR = '0' else
			reg									when DREG = '1' and WR = '0' else
			(others => 'Z');

end Behavioral;
